{-----------------------------------------------------------------------------
 Unit Name: Compiler
 Author:    Patrick Connors
 Purpose:
 Models an ATG file for the purpose of maintaining the GUI in
 the ParserBuilder IDE.

 This is the top-level object.
 Each section of the ATG file is modelled as a child component of this Compiler.


-----------------------------------------------------------------------------}


unit CompilerObj;

interface

uses
  Windows, Messages, SysUtils, Classes, DelphiDeclarations,
  CocoIgnore, CocoProductions, CocoProduction,
  CocoProductionPath, CocoCharacters, CocoTokens,
  pbStrLib, DataFile, CocoPragmas, CocoComments;


type
  TCompilerObjectType =
    (coNone, coCompiler, coHeader, coOptions, coFrames, coComments, coUses,
    coDelphi, coGlobal, coPrivate, coProtected,
    coPublic, coPublished, coCreate, coDestroy, coArbitraryCode,
    coCharacters, coCharacterNode,
    coIgnore, coTokens, coTokenNode, coPragmas, coPragmasNode,
    coProductions, coProductionNode, coTargetDir, coErrors, coNames);

type
  TCompiler = class(TComponent)
  private
    { Private declarations }
    FCharactersSection: TCocoCharacters;
    FIgnoreSection: TCocoIgnore;
    FProductions: TCocoProductions;
    FTokenSection: TCocoTokens;
    FDelphiSection: TDelphiDeclarations;
    FCompilerName: string;
    FErrorFormat: string;
    FLanguageSample: string;
    FFileName: string;
    FPragmasSection: TCocoPragmas;
    fCocoComments: TCocoComments;
    FOriginalATG: string;
    FBuildOptions: string;
    fHeader: string;
    FIsEditing: boolean;
    FIsDirty: boolean;
    FIsEditingHeader: boolean;
    FCurrentEditObject: TCompilerObjectType;
    FCurrentDelphiProject: string;
    FTargetDirectory: string;
    fsaMinIndent: Integer;
    FNames: string;
    FIsEditingNames: boolean;
    procedure SetCharactersSection(const Value: TCocoCharacters);
    procedure SetDelphiSection(const Value: TDelphiDeclarations);
    procedure SetIgnoreSection(const Value: TCocoIgnore);
    procedure SetProductions(const Value: TCocoProductions);
    procedure SetTokenSection(const Value: TCocoTokens);
    procedure SetCompilerName(const Value: string);
    procedure SetLanguageSample(const Value: string);
    procedure SetFileName(const Value: string);
    procedure SetPragmasSection(const Value: TCocoPragmas);
    procedure SetOriginalATG(const Value: string);
    procedure SetBuildOptions(const Value: string);
    function GetCOMPILERFRAME: string;
    function GetPARSERFRAME: string;
    function GetSCANNERFRAME: string;
    procedure SetCOMPILERFRAME(const Value: string);
    procedure SetPARSERFRAME(const Value: string);
    procedure SetSCANNERFRAME(const Value: string);
    function GetErrorFormat: string;
    procedure SetErrorFormat(const Value: string);
    procedure SetCocoComments(const Value: TCocoComments);
    procedure SetHeader(const Value: string);
    procedure SetIsEditing(const Value: boolean);
    procedure SetIsDirty(const Value: boolean);
    procedure SetIsEditingHeader(const Value: boolean);
    procedure SetCurrentEditObject(const Value: TCompilerObjectType);
    procedure SetCurrentDelphiProject(const Value: string);
    procedure SetTargetDirectory(const Value: string);
    procedure SetNames(const Value: string);
    procedure SetIsEditingNames(const Value: boolean);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddProduction(const ProdName, ProdBody: string): TCocoProduction;
    procedure GetProductionNames(SL: TStrings);
    function GetProdNamesList: string;
    function GetATG: string;
    function GetPostEditATG: string;
    procedure ResetAllEditFlags;
    procedure ClearAll;
    procedure SetATGToBeOriginal;
    procedure EditCompilerObject(const cot: TCompilerObjectType; const NodeText: string);
    function GetEditedATG(const Newtext: string): string;
    property DelphiSection: TDelphiDeclarations read FDelphiSection write SetDelphiSection;
    property IgnoreSection: TCocoIgnore read FIgnoreSection write SetIgnoreSection;
    property Productions: TCocoProductions read FProductions write SetProductions;
    property CharactersSection: TCocoCharacters          
      read FCharactersSection write SetCharactersSection;
    property TokenSection: TCocoTokens read FTokenSection write SetTokenSection;
    property PragmasSection: TCocoPragmas read FPragmasSection write SetPragmasSection;
    property CocoComments: TCocoComments read FCocoComments write SetCocoComments;
    property IsDirty: boolean read FIsDirty write SetIsDirty;
    property CurrentEditObject: TCompilerObjectType          
      read FCurrentEditObject write SetCurrentEditObject;
    property CurrentDelphiProject: string read FCurrentDelphiProject          
      write SetCurrentDelphiProject;
  published
    { Published declarations }
    property CompilerName: string read FCompilerName write SetCompilerName;
    property LanguageSample: string read FLanguageSample write SetLanguageSample;
    property FileName: string read FFileName write SetFileName;
    property OriginalATG: string read FOriginalATG write SetOriginalATG;
    property BuildOptions: string read FBuildOptions write SetBuildOptions;
    property COMPILERFRAME: string read GetCOMPILERFRAME write SetCOMPILERFRAME;
    property PARSERFRAME: string read GetPARSERFRAME write SetPARSERFRAME;
    property SCANNERFRAME: string read GetSCANNERFRAME write SetSCANNERFRAME;
    property TargetDirectory: string read FTargetDirectory write SetTargetDirectory;
    property ErrorFormat: string read GetErrorFormat write SetErrorFormat;
    property Header: string read fHeader write SetHeader;
    property IsEditing: boolean read FIsEditing write SetIsEditing;
    property IsEditingHeader: boolean read FIsEditingHeader write SetIsEditingHeader;
    property saMinIndent: Integer read fsaMinIndent write fsaMinIndent;
    property Names: string read FNames write SetNames;
    property IsEditingNames: boolean read FIsEditingNames write SetIsEditingNames;
  end;


implementation

  { TCompiler }

function TCompiler.AddProduction(const ProdName, ProdBody: string): TCocoProduction;
begin
  Result := Productions.AddProduction(ProdName, ProdBody);
end;

procedure TCompiler.ClearAll;
begin
  FCharactersSection.Free;
  FIgnoreSection.Free;
  FProductions.Free;
  FTokenSection.Free;
  FDelphiSection.Free;
  FPragmasSection.Free;
  fCocoComments.Clear;

  FCharactersSection := TCocoCharacters.Create(Self);
  FIgnoreSection := TCocoIgnore.Create(Self);
  FProductions := TCocoProductions.Create(Self);
  FTokenSection := TCocoTokens.Create(Self);
  FDelphiSection := TDelphiDeclarations.Create(Self);
  FPragmasSection := TCocoPragmas.Create(Self);
  FBuildOptions := '';
  FCompilerName := '';
  //  FLanguageSample := '';
  fsaMinIndent := 40; //default min indent for semantic actions
  // Don't clear Original ATG!
end;

constructor TCompiler.Create(AOwner: TComponent);
begin
  inherited;
  FCharactersSection := TCocoCharacters.Create(Self);
  FIgnoreSection := TCocoIgnore.Create(Self);
  FProductions := TCocoProductions.Create(Self);
  FTokenSection := TCocoTokens.Create(Self);
  FDelphiSection := TDelphiDeclarations.Create(Self);
  FPragmasSection := TCocoPragmas.Create(Self);
  fCocoComments := TCocoComments.Create(Self);
  FCompilerName := 'Untitled';
end;

destructor TCompiler.Destroy;
begin
  FCharactersSection.Free;
  FIgnoreSection.Free;
  FProductions.Free;
  FTokenSection.Free;
  FDelphiSection.Free;
  FPragmasSection.Free;
  fCocoComments.Free;
  inherited;
end;

function TCompiler.GetATG: string;
begin
  if not fIsEditing then
  begin
    if FBuildOptions <> '' then
      Result := '$' + FBuildOptions + CRLF
    else
      Result := '';

    Result := Result + 'COMPILER ' + trim(CompilerName) + CRLF;

    FDelphiSection.Filename := FFileName;
    FDelphiSection.TargetDirectory := FTargetDirectory;

    Result := Result + FDelphiSection.GetATG + CRLF;
    if (FIgnoreSection.IgnoreCase) then
      Result := Result + 'IGNORE CASE' + CRLF + CRLF;

    // new 24-04-02
    // Some users may not enter the appropriate section title
    // therefore, we will detect its absence and plug it in.
    // using var s: string ...


    Result := Result + FCharactersSection.GetATG + CRLF;
    Result := Result + FTokenSection.GetATG + CRLF;

    if (fIsEditingNames) then
      Result := Result + 'NAMES' + CRLF + ModTextTag
    else
      Result := Result + 'NAMES' + CRLF + Trim(FNames) + CRLF;

    Result := Result + Trim(FIgnoreSection.GetATG) + CRLF;

    Result := Result + Trim(FPragmasSection.GetATG) + CRLF;

    Result := Result + fCocoComments.GetATG + CRLF;

    fProductions.saMinIndent := fsaMinIndent;
    Result := Result + FProductions.GetATG + CRLF;

    Result := Result + 'END ' + CompilerName + '.' + CRLF;
  end
  else
    Result := ModTextTag;
end;

function TCompiler.GetCOMPILERFRAME: string;
begin
  Result := FDelphiSection.COMPILERFRAME;
end;


function TCompiler.GetErrorFormat: string;
begin
  Result := FErrorFormat;
end;

function TCompiler.GetPARSERFRAME: string;
begin
  Result := FDelphiSection.PARSERFRAME;
end;


procedure TCompiler.GetProductionNames(SL: TStrings);
begin
  FProductions.GetProductionNames(SL);
end;

function TCompiler.GetSCANNERFRAME: string;
begin
  Result := FDelphiSection.SCANNERFRAME;
end;

procedure TCompiler.SetBuildOptions(const Value: string);
begin
  FBuildOptions := Value;
end;

procedure TCompiler.SetCharactersSection(const Value: TCocoCharacters);
begin
  FCharactersSection := Value;
end;

procedure TCompiler.SetCocoComments(const Value: TCocoComments);
begin
  FCocoComments := Value;
end;

procedure TCompiler.SetHeader(const Value: string);
begin
  fHeader := Value;
end;

procedure TCompiler.SetCOMPILERFRAME(const Value: string);
begin
  FDelphiSection.COMPILERFRAME := Value;
end;

procedure TCompiler.SetCompilerName(const Value: string);
begin
  FCompilerName := Trim(Value);
  FProductions.CompilerName := FCompilerName;
end;


procedure TCompiler.SetDelphiSection(const Value: TDelphiDeclarations);
begin
  FDelphiSection := Value;
end;

procedure TCompiler.SetErrorFormat(const Value: string);
begin
  FErrorFormat := Value;
end;

procedure TCompiler.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TCompiler.SetIgnoreSection(const Value: TCocoIgnore);
begin
  FIgnoreSection := Value;
end;

procedure TCompiler.SetLanguageSample(const Value: string);
begin
  FLanguageSample := Value;
end;

procedure TCompiler.SetOriginalATG(const Value: string);
begin
  FOriginalATG := Value;
end;

procedure TCompiler.SetPARSERFRAME(const Value: string);
begin
  FDelphiSection.PARSERFRAME := Value;
end;


procedure TCompiler.SetPragmasSection(const Value: TCocoPragmas);
begin
  FPragmasSection := Value;
end;

procedure TCompiler.SetProductions(const Value: TCocoProductions);
begin
  FProductions := Value;
end;

procedure TCompiler.SetSCANNERFRAME(const Value: string);
begin
  FDelphiSection.SCANNERFRAME := Value;
end;


procedure TCompiler.SetTokenSection(const Value: TCocoTokens);
begin
  FTokenSection := Value;
end;

function TCompiler.GetEditedATG(const Newtext: string): string;
var 
  S: string; // replacement text to be plugged into ATG as an edit.
begin
  // Its assumed that an object has been previously thrown into an edit state
  // via  the EditCompilerObject method
  FIsDirty := True;

  // Clean up the edited text - to remove double entries of
  // the ATG section header (as the user may or may not have entered them, they have
  // been added blindly to what the user has edited).
  S := Trim(NewText);
  case FCurrentEditObject of
    coComments:
      S := ReplaceAll(S, 'COMMENTS' + #13#10, '');
    coCharacters:
      S := ReplaceAll(S, 'CHARACTERS' + #13#10, '');
    coIgnore:
      S := ReplaceFirst(S, 'IGNORE ', '');
    coTokens:
      S := ReplaceAll(S, 'TOKENS' + #13#10, '');
    coPragmas:
      S := ReplaceAll(S, 'PRAGMAS' + #13#10, '');
    coProductions:
      S := ReplaceAll(S, 'PRODUCTIONS' + #13#10, '');
    coNames:
      S := ReplaceAll(S, 'NAMES' + #13#10, '');
  end;// Case


  if (FCurrentEditObject = coHeader) then
    fHEader := S;
  Result := GetATG;
  //  SAveString(Result,'C:\temp\Result.txt');
  if (FCurrentEditObject in [coProductionNode]) then
  begin
    if (S <> '') then
      Result := ReplaceFirst(Result, ModTextTag,
        KillFinalDelim(S, '.') + '.')
    else
      Result := ReplaceFirst(Result, ModTextTag, '');
  end
  else
    Result := ReplaceFirst(Result, ModTextTag, S);


  // if the user has deleted a production, there will be a residual
  // ModTextTag tag left in the ATG - this stuffs up every
  // production after it in the ATG file.
  Result := ReplaceAll(Result, ModTextTag, '');
  //  SAveString(Result,'C:\temp\Result1.txt');
  ResetAllEditFlags;
end;

{-----------------------------------------------------------------------------
  Procedure: TCompiler.EditCompilerObject
  Arguments: const cot: TCompilerObjectType
  Result:    None
  Description

  Flag the selected object as undergoing editing.
  THis will affect the way ATG is produced.

-----------------------------------------------------------------------------}

procedure TCompiler.EditCompilerObject(const cot: TCompilerObjectType;
  const NodeText: string);
begin
  FCurrentEditObject := cot;
  case cot of
    coCompiler:
    begin
      fIsEditing := True;
    end;
    coHeader:
    begin
      FIsEditingHeader := True;
    end;
    coNames:
    begin
      FIsEditingNames := True;
    end;
    coOptions:
    begin
    end;
    coFrames:
    begin
    end;
    coComments:
    begin
      fCocoComments.IsEditing := True;
    end;
    coUses:
    begin
      FDelphiSection.IsEditingUsesClause := True;
    end;
    coDelphi:
    begin
      FDelphiSection.IsEditing := True;
    end;
    coGlobal:
    begin
      FDelphiSection.IsEditingTypeDeclarations := True;
    end;
    coPrivate:
    begin
      FDelphiSection.IsEditingPrivateDecs := True;
    end;
    coErrors:
    begin
      FDelphiSection.IsEditingErrorsSource := True;
    end;
    coProtected:
    begin
      FDelphiSection.IsEditingProtectedDecs := True;
    end;
    coPublic:
    begin
      FDelphiSection.IsEditingPublicDecs := True;
    end;
    coPublished:
    begin
      FDelphiSection.IsEditingPublishedDecs := True;
    end;
    coCreate:
    begin
      FDelphiSection.IsEditingCreateSrc := True;
    end;
    coDestroy:
    begin
      FDelphiSection.IsEditingDestroySrc := True;
    end;
    coArbitraryCode:
    begin
      FDelphiSection.IsEditingArbitraryCodeSection := True;
    end;
    coCharacters:
    begin
      FCharactersSection.IsEditing := True;
    end;
    coCharacterNode:
    begin
      FCharactersSection.EditingCharSet := NodeText;
    end;
    coIgnore:
    begin
      FIgnoreSection.IsEditing := True;
    end;
    coTokens:
    begin
      FTokenSection.IsEditing := True;
    end;
    coTokenNode:
    begin
      FTokenSection.EditingToken := NodeText;
    end;
    coPragmas:
    begin
      FPragmasSection.IsEditing := True;
    end;
    coPragmasNode:
    begin
      FPragmasSection.EditingPragma := NodeText;
    end;
    coProductions:
    begin
      FProductions.IsEditing := True;
    end;
    coProductionNode:
    begin
      FProductions.IsEditingPath := NodeText;
    end;
  end; //Case
end;

{-----------------------------------------------------------------------------
  Procedure: TCompiler.GetPostEditATG
  Arguments: None
  Result:    String
  Description

  Returns the ATG with the new text inserted at the correct place.
-----------------------------------------------------------------------------}

function TCompiler.GetPostEditATG: string;
begin
  ResetAllEditFlags;
  Result := GetATG;
end;

{-----------------------------------------------------------------------------
  Procedure: TCompiler.ResetAllEditFlags
  Arguments: None
  Result:    None
  Description

  Turns off the IsEditing property of all owned objects.
  - this ensures that the next ATG generation wil be a pure reflection
  of the compiler's state - no free text from the GUI will intrude.

  Intended to be used post-editing.

-----------------------------------------------------------------------------}

procedure TCompiler.ResetAllEditFlags;
begin
  FCurrentEditObject := coNone;
  FIsEditing := False;
  FIsEditingHeader := False;
  FIsEditingNames := False;
  FDelphiSection.IsEditing := False;
  FDelphiSection.IsEditingUsesClause := False;
  FDelphiSection.IsEditingTypeDeclarations := False;
  FDelphiSection.IsEditingPrivateDecs := False;
  FDelphiSection.IsEditingTypeDeclarations := False;
  FDelphiSection.IsEditingProtectedDecs := False;
  FDelphiSection.IsEditingPublicDecs := False;
  FDelphiSection.IsEditingPublishedDecs := False;
  FDelphiSection.IsEditingDestroySrc := False;
  FDelphiSection.IsEditingCreateSrc := False;
  FDelphiSection.IsEditingErrorsSource := False;
  FDelphiSection.IsEditingArbitraryCodeSection := False;
  FDelphiSection.IsEditingTargetDir := False;


  FCharactersSection.IsEditing := False;
  FCharactersSection.EditingCharSet := '';
  FIgnoreSection.IsEditing := False;
  FProductions.IsEditing := False;
  FProductions.IsEditingPath := '';
  FTokenSection.IsEditing := False;
  FTokenSection.EditingToken := '';
  FPragmasSection.IsEditing := False;
  FPragmasSection.EditingPragma := '';
  fCocoComments.IsEditing := False;
end;

procedure TCompiler.SetIsEditing(const Value: boolean);
begin
  FIsEditing := Value;
end;

procedure TCompiler.SetIsDirty(const Value: boolean);
begin
  FIsDirty := Value;
end;

procedure TCompiler.SetIsEditingHeader(const Value: boolean);
begin
  FIsEditingHeader := Value;
end;

procedure TCompiler.SetCurrentEditObject(const Value: TCompilerObjectType);
begin
  FCurrentEditObject := Value;
end;

procedure TCompiler.SetCurrentDelphiProject(const Value: string);
begin
  FCurrentDelphiProject := Value;
end;

procedure TCompiler.SetTargetDirectory(const Value: string);
begin
  FTargetDirectory := Value;
end;

procedure TCompiler.SetATGToBeOriginal;
begin
  //
end;

procedure TCompiler.SetNames(const Value: string);
begin
  FNames := Value;
end;

procedure TCompiler.SetIsEditingNames(const Value: boolean);
begin
  FIsEditingNames := Value;
end;

function TCompiler.GetProdNamesList: string;
var 
  SL: TStringList;
begin
  SL := TStringList.Create;
  FProductions.GetProductionNames(SL);
  Result := ReplaceAll(SL.Text, #13#10, '');
  SL.Free;
end;


end.
